package App::Acmeman::Apache::Layout;

use strict;
use warnings;
use Carp;
use File::Basename;
use feature 'state';
use App::Acmeman::Log qw(:all);

sub _find_httpd {
    my $httpd;
    foreach my $d (split /:/, $ENV{PATH}) {
	foreach my $n (glob "$d/apachectl $d/httpd") {
	    if (-x $n) {
		return $n;
	    }
	}
    }
}

sub modules {
    my $class = shift;
    my @path = split /::/, $class;
    
    state $loaders //=
	[map { $_->[1] }
	 sort { $a->[0] <=> $b->[0] }
	 map {
	     my ($modname) = $class . '::' . fileparse($_, '.pm');
	     eval {
		 no strict 'refs';
		 if (scalar %{ $modname.'::' }) {
		     ()
		 } else {
		     require $_;
		     my $prio = ${$modname.'::PRIORITY'} // 0;
		     [ $prio, $modname ]
	         }
	     };
	 }
	 map { glob File::Spec->catfile($_, '*.pm') }
	 grep { -d $_ }
	 map { File::Spec->catfile($_, @path) } @INC];
    @$loaders;
}

# new(NAME)
# new()
sub new {
    my $class = shift;
    
    if (@_ == 0) {
	# Autodetect
	debug(3, "detecting Apache configuration layout");
	my $ap = new Apache::Defaults(server => $class->_find_httpd,
				      on_error => 'return');
	if ($ap->status) {
	    croak "unable to get Apache defaults: " . $ap->error;
	}
	
	foreach my $mod ($class->modules) {
	    debug(3, "trying layout module $mod");
	    my $obj;
	    eval {
		$obj = $mod->new($ap);
	    };
	    if ($obj) {
		return $obj;
	    }
	    if ($@) {
		debug(3, "layout module failed: $@");
	    }
	}

	if ($ap) {
	    return new App::Acmeman::Apache::Layout::auto($ap);
	}

	croak "unrecognized Apache layout";
    }

    my ($name, %args) = @_;
    if (ref($name) eq '') {
	my $mod = "$class::$name";
	my $obj;
	eval {
	    require $mod;
	};
	croak "undefined Apache layout $name" if ($@);
	return $mod->new;
    }

    unless ($name->isa('Apache::Defaults')) {
	croak "unrecognized argument";
    }

    my $self = bless { _defaults => $name }, $class;
    foreach my $kw (qw(layout_name incdir restart_command)) {
	if (defined(my $v = delete $args{$kw})) {
	    $self->{$kw} = $v;
	}
    }

    return $self;
}    

sub apache { shift->{_defaults} }

sub name {
    my $self = shift;
    unless ($self->{layout_name}) {
	$self->{layout_name} = ref($self);
	$self->{layout_name} =~ s/.*:://;
	$self->{layout_name} =~ s/\.pm$//;
    }
    return $self->{layout_name};
}

sub config_file { shift->apache->server_config }

sub incdir {
    my $self = shift;
    unless ($self->{incdir}) {
	$self->{incdir} = dirname($self->config_file);
    }
    return $self->{incdir};
}

sub server_command { join(' ', shift->apache->server_command) }

sub restart_command {
    my $self = shift;

    unless (exists($self->{restart_command})) {
	if ($self->server_command =~ m{/apachectl$}) {
	    $self->{restart_command} = $self->server_command . ' restart';
	} else {
	    error("no postrenew command defined", prefix => 'warning');
	    $self->{restart_command} = undef
	}
    }
    return $self->{restart_command};
}

sub post_setup {}

package App::Acmeman::Apache::Layout::auto;
use parent 'App::Acmeman::Apache::Layout';
use App::Acmeman::Log qw(:all);

sub new {
    my ($class, $ap) = @_;
    return $class->SUPER::new($ap);
}

1;
